package de.kif.frontend.views.overview

import de.kif.frontend.iterator
import de.kif.frontend.launch
import de.kif.frontend.repository.PostRepository
import de.westermann.kobserve.event.subscribe
import org.w3c.dom.HTMLElement
import org.w3c.dom.HTMLTextAreaElement
import org.w3c.dom.events.EventListener
import org.w3c.dom.get
import kotlin.browser.document
import kotlin.dom.clear

private fun sortOverviewPosts(container: HTMLElement) {
    val list = container.children.iterator().asSequence().toList()

    val sorted = list.sortedWith(compareBy(
        { if (it.dataset["pinned"]?.toBoolean() == true) 0 else 1 },
        { -(it.dataset["id"]?.toLong() ?: -1) }
    ))

    container.clear()

    for (element in sorted) {
        container.appendChild(element)
    }
}

fun initOverviewMain() {
    val main = document.getElementsByClassName("overview-main")[0] as HTMLElement

    PostRepository.onCreate {
        val post = PostView.create(it)
        post.classList += "overview-post"
        main.appendChild(post.html)

        sortOverviewPosts(main)
    }

    subscribe<PostChangeEvent> {
        sortOverviewPosts(main)
    }
}

fun initPosts() {
    val postList = document.getElementsByClassName("post")
    for (post in postList) {
        PostView(post)
    }
}

fun initPostEdit() {
    val textArea = document.getElementById("content") as HTMLTextAreaElement
    val preview = document.getElementsByClassName("post-edit-right")[0] as HTMLElement

    textArea.addEventListener("change", EventListener {
        launch {
            preview.innerHTML = PostRepository.render(textArea.value)
        }
    })
    textArea.addEventListener("keyup", EventListener {
        launch {
            preview.innerHTML = PostRepository.render(textArea.value)
        }
    })

    launch {
        preview.innerHTML = PostRepository.render(textArea.value)
    }
}