package de.kif.frontend.views.overview

import de.kif.frontend.launch
import de.kif.frontend.repository.PostRepository
import de.westermann.kobserve.event.emit
import de.westermann.kwebview.View
import de.westermann.kwebview.components.Link
import de.westermann.kwebview.createHtmlView
import org.w3c.dom.HTMLAnchorElement
import org.w3c.dom.HTMLElement
import org.w3c.dom.get
import org.w3c.dom.set
import kotlin.browser.document

class PostView(
    view: HTMLElement
) : View(view) {

    val postId = dataset["id"]?.toLongOrNull() ?: -1

    var pinned: Boolean
        get() = dataset["pinned"] == "true"
        set(value) {
            dataset["pinned"] = value.toString()
        }

    private val nameView: Link
    private val contentView: View

    private fun reload() {
        launch {
            val p = PostRepository.get(postId) ?: return@launch

            nameView.text = p.name
            nameView.target = "/p/${p.url}"
            pinned = p.pinned

            contentView.html.innerHTML = PostRepository.htmlByUrl(p.url)

            emit(PostChangeEvent(postId))
        }
    }

    init {
        val nameHtml = view.getElementsByClassName("post-name")[0]
        nameView = nameHtml?.let { Link.wrap(it as HTMLAnchorElement) } ?: Link().also {
            html.appendChild(it.html)
            it.classList += "post-name"
        }

        // val editHtml = view.getElementsByClassName("post-edit")[0]
        // editView = editHtml?.let { Link.wrap(it as HTMLAnchorElement) } ?: Link()

        val contentHtml = view.getElementsByClassName("post-content")[0]
        contentView = contentHtml?.let { wrap(it as HTMLElement) } ?: wrap(createHtmlView()).also {
            html.appendChild(it.html)
            it.classList += "post-content"
        }

        PostRepository.onUpdate {
            if (it == postId) {
                reload()
            }
        }
        PostRepository.onDelete {
            html.remove()
        }
    }

    companion object {
        fun create(postId: Long): PostView {
            val div = document.createElement("div") as HTMLElement
            div.classList.add("post")
            div.dataset["id"] = postId.toString()
            return PostView(div).also(PostView::reload)
        }
    }
}

data class PostChangeEvent(val id: Long)
